%THIS SCRIPT IMPORTS ALONG THE SHORTEST ROUTE
clear all

load ('undirected.txt');

network = undirected;
adj_net = edgeL2adj(network);
sparse_net = sparse(adj_net);

L=size(sparse_net,1);
d=zeros(L,L);sortd=d;indexd=d;
pred=zeros(L,L);
shortroot=zeros(L,L,L)+NaN;

for i=1:L
    [d(:,i),pp]=dijkstra(sparse_net, i);
    pred(i,:)=pp;
    [sortd(:,i),indexd(:,i)]=sort(d(:,i));
    for m =1:i-1
        path =[]; u = m; while (u ~= i) path=[u path]; u=pred(i,u); end
        % FIRST DIM OF SHORTROOT IS START NODE; SECOND DIMENSION IS END
        % NODE; ON THE THIRD DIMENSION IS THE PATH (AND MAY BE A LOT OF
        % NANs)
        shortroot(i,m,1:length(path))=path; 
    end
    for m =i+1:L
        path =[]; u = m; while (u ~= i) path=[u path]; u=pred(i,u); end
        % FIRST DIM OF SHORTROOT IS START NODE; SECOND DIMENSION IS END
        % NODE; ON THE THIRD DIMENSION IS THE PATH (AND MAY BE A LOT OF
        % NANs)
        shortroot(i,m,1:length(path))=path; 
    end
end

% TO EXTRACT A PATH FROM NODE 30 TO 400:
aaa=squeeze(shortroot(2,632,:));
aaa=aaa(isfinite(aaa));

% TO SAVE THE IMPORTANT DATA

% % SAVE THE SHORTEST ROUTES
save('short_root_test.mat','shortroot','-v7.3')
% 
% %SAVE THE DISTANCE LOOKUP TABLE
% 
save('dist_lookup.mat','d','-v7.3')
% 
% %SAVE THE SORTED DISTANCES (I.E. DIST FROM NEAREST TO FURTHEST)
% 
save('dist_closest_to_furthest.mat','sortd','-v7.3')
% 
% %SAVE THE INDEX OF THE SORTED DISTANCES(I.E. NEAREST TO FURTHEST NODES)
% 
save ('node_closest_to_furthest.mat','indexd','-v7.3')
% 
clear all

%LOAD IN THE RELEVANT DATASETS

load short_root_test.mat
load dist_lookup.mat
load dist_closest_to_furthest.mat
load node_closest_to_furthest.mat
load sur_def_b4_trade.txt

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%% EXPLANATION OF IMPORTED DATA%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%THE LOADED DATA ARE D, INDEXD, SHORTROOT, SORTD AND THE NODE LIST WITH
%VALUES

% D IS THE DISTANCE MATRIX GIVING THE SHORTEST DISTANCE FROM EACH TO EVERY
% OTHER NODE

% SORTD IS THE SAME AS D BUT SORTED FROM CLOSEST TO FARTHEST AWAY

% INDEXD TELLS WHICH NODE IS ASSOCIATED WITH WHICH DISTANCE IN THE SORTD
% MATRIX

% SHORTROOT GIVES THE SHORTEST PATHWAY FROM EACH NODE TO EVERY OTHER NODE.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%SET THE NUMBER OF ITERATIONS
iterations = 1000;

sdnl = sur_def_b4_trade;

%%%%%%%%%%%%%%%% MAKE POPULATION ADJUSTMENTS OF CHANDLER %%%%%%%%%%%%%%%%%%
sdnl(230,2) = -94400000; %Rome
sdnl(172,2) = -61353000; %Alexandria
sdnl(30,2) = -35471200;  %Ephesus
sdnl(211,2) = -25480000; %Antioch
sdnl(635,2) = -20082800; %Apamea
sdnl(515,2) = -14400000; %Pergamum
sdnl(294,2) = -12224000; %Cadiz
sdnl(485,2) = -7030400;  %Corinth
sdnl(427,2) = -16362600; %Sardis
sdnl(292,2) = -16400000; %Syracuse
sdnl(516,2) = -15000000; %Symrna
sdnl(338,2) = -8000000;  %Cordova
sdnl(629,2) = -8000;     %Caesarea
sdnl(647,2) = -8800000;  %Volulibis
sdnl(373,2) = -7600000;  %Autun
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

sdnl_orig = sdnl;

sdnl_pos = find(sdnl_orig(:,2) > 0);
sdnl_neg = find(sdnl_orig(:,2) < 0);

% min_val = -1 * (max(abs(sdnl_orig(sdnl_neg,2))) / iterations);
max_val = max(sdnl_orig(sdnl_pos,2)) / iterations;

J = size(sdnl,1);
K = size(sdnl,1);

%DEFINE THIS TO INITIATE LOOP
total_deficit = -1;
%   INITIATE SOME MATRICES
flow_matrix = zeros(J,J);
sur_def_mat_orig = zeros(J,J);

surplus_nodes_orig = find(sdnl(:,2) > 0);
deficit_nodes_orig = find(sdnl(:,2) < 0);

%THIS CREATES A MATRIX WITH VALUES AT NEAREST TO FURTHEST NODES
for jj = 1:J
    for kk = 1:K
        sur_def_mat_orig(kk,jj) = sdnl_orig(indexd(kk,jj),2);
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%LOOP THE PROGRAM WHILE THERE REMAINS A NODE WITH A DEFICIT%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      while total_deficit < 0
         

%THIS CREATES A MATRIX WITH VALUES AT NEAREST TO FURTHEST NODES THAT IS
%UPDATED EACH ITERATION
sur_def_mat = zeros(J,J);

for jj = 1:J
    for kk = 1:K
        sur_def_mat(kk,jj) = sdnl(indexd(kk,jj),2);
    end
end


%FIND NODES WITH A SURPLUS
surplus_nodes = zeros(J,J)+NaN;
deficit_nodes = zeros(J,J)+NaN;

for jj = 1:J
     surplus_nodes(1:length(find(sur_def_mat(:,jj) > 0)),jj) = find(sur_def_mat(:,jj) > 0);
     deficit_nodes(1:length(find(sur_def_mat(:,jj) < 0)),jj) = find(sur_def_mat(:,jj) < 0);
end

% PLACE THE SURPLUS AND THE DEFICIT NODES IN A SEPARATE MATRIX
def_orig = sur_def_mat_orig;
def_orig(find(def_orig > 0)) = 0;
sur_orig = sur_def_mat_orig;
sur_orig(find(sur_orig < 0)) = 0;

%THIS MEANS THAT THE PROGRAMME KNOWS THAT SURPLUS NODES REMAIN SURPLUS
%EXPORT NODES EVEN AFTER THEIR SURPLUS IS DEPLETED.
surplus_nodes(:,surplus_nodes_orig) = 1;

sdnl_cum_demand = zeros(J,J);

% THIS SHOWS THE FREQUENCY OF FLOWS ALONG EACH PATH
prelim_flow_matrix = zeros(J,J);

for ii = 1:J
    x = squeeze(shortroot(ii,indexd(surplus_nodes(1,ii),ii),:));
    x = x(isfinite(x));
    eval(['node_',int2str(ii),'_path=x;'])

    if isempty(x)==0
        %THE COMMENTED CODE GIVES THE FREQUENCY OF TRADE RATHER THAN THE
        %ACTUAL AMOUNT TRADED
        
%         flow_matrix(ii,x(1))=flow_matrix(ii,x(1))+1;
        flow_matrix(ii,x(1))=flow_matrix(ii,x(1))+(-1* (sdnl_orig(ii,2)/iterations));

        for i=1:length(x)-1
%             flow_matrix(x(i),x(i+1))=flow_matrix(x(i),x(i+1))+1;
            flow_matrix(x(i),x(i+1))=flow_matrix(x(i),x(i+1))+(-1* (sdnl_orig(ii,2)/iterations));

        end
    end
 
end
        
% THE DEFICIT CELL SUBSTRACTED FROM THE SURPLUS CELL
    
% FOR EACH DEFICIT NODE WE FIND NEAREST SURPLUS NODE. WE SEARCH THE INDEX
% FOR THE NODE ID OF THAT SURPLUS NODE. WE THEN FIND THAT NODE'S VALUES IN
% THE SDNL AND TAKE THE ORIGINAL DEFICIT AWAY

for jj = 1:J
    
sdnl_cum_demand(jj,indexd(surplus_nodes(1,jj),jj)) = (def_orig(1,jj) / iterations);
sdnl_cum_demand(jj,jj) = 0;

end

%SDNL_CUM_DEM IS THE SUM OF ALL DEMAND ON A PARTICULAR NODE
sdnl_cum_dem = sum(sdnl_cum_demand);
sdnl(:,2) = sdnl(:,2) + sdnl_cum_dem';

%ADD IMPORTS TO EACH DEFICIT NODE
sdnl(deficit_nodes_orig,2) = sdnl(deficit_nodes_orig,2) + ( -1 *  (sdnl_orig(deficit_nodes_orig,2) / iterations));

%CALCULATE THE TOTAL DEFICIT
total_deficit = sum(sdnl(deficit_nodes_orig,2));
 
% sdnl
% sum(sdnl(:,2))
 clearvars -except J K d indexd sdnl sdnl_orig shortroot sortd total_deficit sur_def_mat_orig deficit_nodes_orig iterations flow_matrix surplus_nodes_orig  

       end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% END LOOP %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
trade_flows = adj2edgel(flow_matrix)

save 'link_result_matlab_no_chandler.txt' trade_flows -ascii
save 'surplus_deficit.txt' sdnl -ascii

